export type AutoProxyObj<T extends Record<string, any>> = {
	[K in keyof T]: AutoProxy<T[K]>;
};

export type AutoProxyArr<T> = {
	[key: number]: AutoProxy<T>;
};

export type AutoProxy<T> = T extends Record<string, any>
	? AutoProxyObj<Required<T>>
	: T extends (infer U)[]
	? AutoProxyArr<Required<U>>
	: T;

export const autoProxy = <T>(initialValue?: T): AutoProxy<T> => {
	let self: undefined | Record<string, any> | any[] = initialValue as any;

	const p = new Proxy(
		{},
		{
			get: (_, prop, __) => {
				if (prop == "___self") {
					return self;
				}

				if (self == undefined) {
					if (Number.isNaN(Number(prop))) {
						self = {};
					} else {
						self = [];
					}
				}

				if (!Object.hasOwn(self, prop)) {
					// @ts-ignore:
					self[prop] = autoProxy();
				}

				// @ts-ignore:
				return self[prop];
			},

			set: (_, prop, value, __) => {
				if (self == undefined) {
					if (Number.isNaN(prop)) {
						self = {};
					} else {
						self = [];
					}
				}

				if (!Object.hasOwn(self, prop)) {
					// @ts-ignore:
					self[prop] = autoProxy();
				}

				// @ts-ignore:
				self[prop] = value;

				return true;
			},
			ownKeys: () => Object.keys(self || {}),
		},
	);

	return p as any;
};

export const unproxy = <T>(proxy: AutoProxy<T>): T => {
	const self = (proxy as any).___self;
	if (!self) {
		return undefined as any;
	}

	return Object.fromEntries(
		Object.entries(self).map(([k, v]) => {
			if ((v as any)["___self"]) {
				return [k, unproxy(v)];
			}

			return [k, v];
		}),
	) as T;
};
