import { Const } from "../../common/utils.ts";

export type ShouldPaint<Model> = (model: Const<Model>) => boolean;
export type Paint<Model> = (model: Const<Model>) => void;

export abstract class Component<El = HTMLElement> {
	abstract get el(): El;
}

export abstract class ModelComponent<
	Model = never,
	El = HTMLElement,
> extends Component<El> {}

export abstract class PaintComponent<
	Model = never,
	El = HTMLElement,
> extends ModelComponent<El> {
	abstract paint: Paint<Model>;
}

export abstract class CmpPaintComponent<
	Model = never,
	El = HTMLElement,
> extends PaintComponent<Const<Model>, El> {
	protected previousModel: Model | undefined;

	paint: Paint<Model> = (model) => {
		if (this.shouldPaint(model)) {
			this.paint_(model);
		}

		this.previousModel = model as any;
	};

	protected shouldPaint: ShouldPaint<Model> = (cur) => {
		return cur != this.previousModel;
	};

	protected abstract paint_: Paint<Model>;
}

export type ComponentModel<T extends ModelComponent<any>> =
	T extends ModelComponent<infer M> ? Const<M> : never;
