import { swel } from "../deps/swel.ts";

export class RecyclerList<View, Model> {
	private activeViews = 0;
	private _views: View[] = [];
	private container: HTMLElement;

	constructor(
		private createView: () => View,
		private getViewDOMElement: (view: View) => HTMLElement,
		private bindModel: (view: View, model: Model, index: number) => void,
		container?: HTMLElement,
	) {
		this.container = container ?? swel("div", { className: "recycler-list" });
	}

	get el() {
		return this.container;
	}

	changed(models: Model[]) {
		if (this.activeViews < models.length) {
			let i = 0;
			for (; i < this.activeViews; ++i) {
				const view = this._views[i];
				this.bindModel(view, models[i], i);
			}

			for (; i < models.length; ++i) {
				const view = this.createView();
				this._views.push(view);
				this.bindModel(view, models[i], i);

				this.container.appendChild(this.getViewDOMElement(view));
			}
		} else {
			let i = 0;
			for (; i < models.length; ++i) {
				const view = this._views[i];
				this.bindModel(view, models[i], i);
			}

			for (; i < this.activeViews; ++i) {
				this.container.removeChild(this.container.childNodes.item(i));
			}
		}

		this.activeViews = models.length;
	}
}
