import { Const } from "../../../common/utils.ts";

export type OnChange<State, Returning = void> = (
	current: Const<State>,
	prev?: Const<State>,
) => Returning;
export type Reduce<State, Actions> = (
	current: Const<State>,
	action: Actions,
) => Const<State>;
export type Dispatch<Actions> = (action: Actions) => void;

export type Store<Actions> = {
	dispatch: Dispatch<Actions>;
};

export const createStore = <State, Actions>(
	initialState: Const<State>,
	reduce: Reduce<State, Actions>,
	onChange: OnChange<State>,
): Store<Actions> => {
	let dirty = true;
	let prevState: Const<State> | undefined = undefined;
	let state = initialState;

	const doChange = () => {
		console.assert(dirty);
		dirty = false;

		onChange(state, prevState);
		prevState = state;
	};
	requestAnimationFrame(doChange);

	const dispatch = (action: Actions) => {
		if (!dirty) {
			dirty = true;
			requestAnimationFrame(doChange);
		}

		state = reduce(state, action);
	};

	return { dispatch };
};
